import bpy
from bpy.types import Operator, Panel, Menu
from bpy.props import BoolProperty

bl_info = {
    "name": "Hair Tools",
    "author": "3dbi",
    "version": (0, 6, 0),
    "blender": (2, 76, 0),
    "location": "View3D > Tools > Hair Tools Panel",
    "description": "Make Single User Hair Particle Systems And/Or Copy Hair Edits",
    "warning": "",
    "wiki_url": "",
    "category": "Object"
}

class OP_duplicate_active_particle_system(Operator):
    """Copy active particle system and make single user."""
    bl_idname = "particle.duplicate_active_particle_system"
    bl_label = "Duplicate Active Particle System"

    @classmethod
    def poll(cls, context):
        ob = context.active_object
        return True if ob != None and ob.particle_systems.active != None and \
                    len(context.selected_objects) <= 1 and \
                    context.mode in ['OBJECT', 'PARTICLE'] else False

    def execute(self, context):
        print("running...")
        scene = context.scene

        original_ob = context.active_object
        mode = context.mode if context.mode == 'OBJECT' else 'PARTICLE_EDIT'

        if mode != 'OBJECT':
            bpy.ops.object.mode_set(mode='OBJECT')

        bpy.ops.object.duplicate(linked=True)

        ob_copy = context.active_object

        original_ob.select = True
        scene.objects.active = ob_copy

        source_ps = original_ob.particle_systems.active

        #http://blender.stackexchange.com/questions/15307/scripting-cant-figure-out-how-to-use-correct-contexts
        override={'object':ob_copy}
        bpy.ops.particle.copy_particle_systems(override, remove_target_particles=False, use_active=True)

        ps_list = original_ob.particle_systems
        ps_total = len(ps_list)
        ps_list[ps_total-1].settings = ps_list[ps_total-1].settings.copy()
        ps_list.active_index = ps_total-1

        target_ps = original_ob.particle_systems[ps_total-1]

        for j in range(0, len(source_ps.particles)):
            sp = source_ps.particles[j]
            tp = target_ps.particles[j]

            hair_keys_count = len(sp.hair_keys)
            for k in range(0, hair_keys_count):
                tp.hair_keys[k].co = sp.hair_keys[k].co

            # hair_keys seems to return a value one less then the actually number of hair keys 
            # for any hairs added/created in particle edit mode as well as the coordinates seem 
            # to be very slightly off for some reason. I'm using 1.76b so maybe the problem has 
            # been corrected by now. Anyway this is a quick and dirty way to set those hair keys.
            try:
                tp.hair_keys[k+1].co = sp.hair_keys[k].co
            except:
                pass

        original_ob.data.update()

        # Cleanup
        scene.objects.unlink(ob_copy)
        bpy.data.objects.remove(ob_copy)
        scene.objects.active = original_ob
        bpy.ops.object.mode_set(mode=mode)

        return {'FINISHED'}


class OP_copy_active_particle_system_and_make_single_user(Operator):
    """Copy active particle system and make single user."""
    bl_idname = "particle.copy_active_particle_system_and_make_single_user"
    bl_label = "Copy Active Particle System and Make Single User"
    #bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        ob = context.active_object
        return True if ob != None and \
                    ob.particle_systems.active != None and \
                    context.mode in ['OBJECT', 'PARTICLE'] else False

    def execute(self, context):
        scene = context.scene
        active_ob = context.active_object
        source_ps = active_ob.particle_systems.active
        bpy.ops.object.particle_system_add()
        target_ps = active_ob.particle_systems.active
        target_ps.settings = source_ps.settings.copy()

        mode = context.mode if context.mode == 'OBJECT' else 'PARTICLE_EDIT'

        # Don't really understand why this is required here but brush_edit operator will 
        # throw a poll error if it isn't set and this operator is called while in 
        # particle edit mode.
        bpy.ops.object.mode_set(mode='OBJECT') 

        if source_ps.settings.type == 'HAIR':
            target_ps.settings.count = len(source_ps.particles)
            bpy.ops.object.mode_set(mode='PARTICLE_EDIT', toggle=False)
            scene.tool_settings.particle_edit.tool = 'COMB'
            bpy.ops.particle.brush_edit(stroke=[{"name":"", 
                                                 "location":(0, 0, 0), 
                                                 "mouse":(289, 312), 
                                                 "pressure":0, 
                                                 "size":0, 
                                                 "pen_flip":False, 
                                                 "time":0, 
                                                 "is_start":False}])
            bpy.ops.object.mode_set(mode='OBJECT')

            for j in range(0, len(source_ps.particles)):
                sp = source_ps.particles[j]
                tp = target_ps.particles[j]

                hair_keys_count = len(sp.hair_keys)
                for k in range(0, hair_keys_count):
                    tp.hair_keys[k].co = sp.hair_keys[k].co

                # hair_keys seems to return a value one less then the actually number of hair keys 
                # for any hairs added/created in particle edit mode as well as the coordinates seem 
                # to be very slightly off for some reason. I'm using 1.76b so maybe the problem has 
                # been corrected by now. Anyway this is a quick and dirty way to set those hair keys.
                try:
                    tp.hair_keys[k+1].co = sp.hair_keys[k].co
                except:
                    pass

            active_ob.data.update()

        bpy.ops.object.mode_set(mode=mode)
        return {'FINISHED'}


class OP_make_single_user_hair_particle_system(Operator):
    """Make all hair particle systems on selected objects single user or optional only \
effect the active particle system on each selected object."""
    bl_idname = "object.make_single_user_hair_particle_systems"
    bl_label = "Make Single User Hair Particle Systems"
    bl_options = {'REGISTER', 'UNDO'}
    
    only_active = BoolProperty(name="Active Hair Particle System(s) Only",
                               description="Only effect the active hair particle system on each selected object",
                               default=False)

    @classmethod
    def poll(cls, context):
        ob = context.active_object
        return True if ob != None and len(ob.particle_systems) else False

    def execute(self, context):
        selected = bpy.context.selected_objects
        sel_obs = [ob for ob in selected if len(ob.particle_systems)]
        active_ob = context.active_object

        for ob in sel_obs:
            if self.only_active:
                if active_ob != None and active_ob.particle_systems.active != None:
                    ps = active_ob.particle_systems.active
                    if ps.settings.users != 1 and ps.settings.type == 'HAIR':
                        ps.settings = ps.settings.copy() 
            else:
                for ps in ob.particle_systems:
                    if ps.settings.users != 1 and ps.settings.type == 'HAIR':
                        ps.settings = ps.settings.copy()

        return {'FINISHED'}


class OP_copy_hair_edits(Operator):
    """Copy hair style edits from active object to all other selected objects. \
This is very basic and may not work so use at your own risk."""

    bl_idname = "object.copy_hair_edits"
    bl_label = "Copy Hair Edits from Active to Selected"

    def execute(self, context):
        selected = bpy.context.selected_objects 

        active_ob = bpy.context.active_object
        sel_obs = [o for o in selected if o != active_ob]

        for i in range(0, len(active_ob.particle_systems)):

            source_ps = active_ob.particle_systems[i]

            for sel_ob in sel_obs:
                target_ps = sel_ob.particle_systems[i]

                p_total = min(len(source_ps.particles), len(target_ps.particles))

                for j in range(0, p_total):
                    sp = source_ps.particles[j]
                    tp = target_ps.particles[j]

                    hair_keys_count = len(sp.hair_keys)
                    for k in range(0, hair_keys_count):
                        tp.hair_keys[k].co = sp.hair_keys[k].co

                sel_ob.data.update()

        context.scene.update()
        return {'FINISHED'}


class PARTICLE_MT_specials(Menu):
    bl_label = "Particle Specials"
    COMPAT_ENGINES = {'BLENDER_RENDER', 'BLENDER_GAME'}

    def draw(self, context):
        layout = self.layout

        props = layout.operator("particle.copy_particle_systems", text="Copy Active to Selected Objects")
        props.use_active = True
        props.remove_target_particles = False

        props = layout.operator("particle.copy_particle_systems", text="Copy All to Selected Objects")
        props.use_active = False
        props.remove_target_particles = True

        layout.operator("particle.duplicate_active_particle_system", text="Duplicate Particle System")


"""
from bl_ui.properties_particle import PARTICLE_MT_specials

class PARTICLE_MT_specials_extras(PARTICLE_MT_specials):
    bl_idname = "PARTICLE_MT_specials"

    def draw(self, context):
        super(PARTICLE_MT_specials_extras, self).draw(context)
        layout = self.layout
        layout.operator("particle.duplicate_active_particle_system", text="Duplicate Particle System")
"""

class VIEW3D_PT_hair_tools(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'

    bl_context = "objectmode"
    bl_category = "Tools"
    bl_label = "Hair Tools"

    def draw(self, context):

        layout = self.layout
        col = layout.column()
        
        #col.operator("particle.copy_active_particle_system_and_make_single_user", text="Duplicate Particle System")

        col.label(text="Old Tools")
        box = col.box()
        props = box.operator("particle.copy_particle_systems")
        props.remove_target_particles = True
        props.use_active = False

        box.operator("object.make_single_user_hair_particle_systems", text="Make Single User")
        box.operator("object.copy_hair_edits", text="Copy Hair Edits")


def register():
    bpy.utils.register_module(__name__)

def unregister():
    bpy.utils.unregister_module(__name__)

if __name__ == '__main__':
    register()